require 'rails_helper'

module Store
  describe Api::Store::CustomersController, type: :api do
    routes {Store::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:admin) { create(:administrator) }

    let!(:responsible) { create(:user) }
    let!(:user) { create(:user) }

    let!(:customer1) { create(:customer, user: :admin, writers: [responsible.id.to_s]) }
    let!(:customer2) { create(:customer, user: :admin) }
    let!(:customer3) { create(:customer, user: :admin, writers: [responsible.id.to_s]) }
    let!(:customer4) { create(:customer, user: :admin, writers: [responsible.id.to_s]) }

    describe 'GET /customers' do
      context 'as admin' do
        before :each do
          authenticated_header(request, admin)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty

          expect(json['data'].length).to eq(4)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(4)
        end
      end
    end

    describe 'GET /customers?ids' do
      before :each do
        authenticated_header(request, admin)
        get 'index', ids: [customer1.id.to_s, customer2.id.to_s, customer4.id.to_s]
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty

        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end

    describe 'GET /customers/:id' do
      context 'as responsible' do
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: customer1.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['id']).to eq(customer1.id.to_s)
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['title']).to eq(customer1.title)
        end
      end
    end

    describe 'POST /customers' do
      context 'as admin' do
        let(:valid_params) do
          { data: { attributes: {
              title: 'Best Customer'
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          post 'create', valid_params
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(201)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['title']).to eq('Best Customer')
          expect(json['data']['attributes']['responsible']).to eq(admin.id.to_s)
        end
      end
    end

    describe 'PATCH /customers/:id' do
      context 'as admin' do
        let(:valid_params) do
          { id: customer1.id.to_s, data: { attributes: {
              title: 'Best Customer'
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          patch 'update', valid_params
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['id']).to eq(customer1.id.to_s)
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['title']).to eq('Best Customer')
        end
      end
    end
  end
end