require 'rails_helper'

module Store
  describe Api::Store::OrdersController, type: :api do
    routes {Store::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:admin) { create(:administrator) }

    let!(:responsible) { create(:user) }
    let!(:user) { create(:user) }

    let!(:product1) { create(:product, package: 'emails', price: 100) }
    let!(:product2) { create(:product, package: 'files', price: 200) }
    let!(:customer1) { create(:customer) }
    let!(:customer2) { create(:customer) }
    #:id, :currency, :total_price, :host, :payment_amount, :payment_method, :payment_status
    let!(:order1) { create(:order, user: :admin, writers: [responsible.id.to_s], product_ids: [product1.id, product2.id], customer: customer1) }
    let!(:order2) { create(:order, user: :admin, product_ids: [product1.id], customer: customer1) }
    let!(:order3) { create(:order, user: :admin, writers: [responsible.id.to_s], product_ids: [product1.id], customer: customer2) }
    let!(:order4) { create(:order, user: :admin, writers: [responsible.id.to_s], product_ids: [product1.id], customer: customer2) }

    describe 'GET /orders' do
      context 'as admin' do
        before :each do
          authenticated_header(request, admin)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty

          expect(json['data'].length).to eq(4)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(4)
        end
      end
    end

    describe 'GET /orders?ids' do
      before :each do
        authenticated_header(request, admin)
        get 'index', ids: [order1.id.to_s, order2.id.to_s, order4.id.to_s]
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty

        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end

    describe 'GET /orders?customer_id' do
      before :each do
        authenticated_header(request, admin)
        get 'index', customer_id: customer1.id.to_s
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty

        expect(json['data'].length).to eq(2)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(2)
      end
    end

    describe 'GET /orders/:id' do
      context 'as responsible' do
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: order1.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['id']).to eq(order1.id.to_s)
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['payment_status']).to eq('not_processed')
        end
      end
    end

    describe 'POST /orders' do
      context 'as admin' do
        let(:valid_params) do
          { data: { attributes: {
              product_ids: [product1.id, product2.id]
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          post 'create', valid_params
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(201)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['user_id']).to eq(admin.id.to_s)
          expect(json['data']['relationships']['products']['data'].length).to eq(2)
          expect(json['data']['relationships']['products']['data'][0]['id']).to eq(product1.id.to_s)
        end
      end
    end

    describe 'PATCH /orders/:id' do
      context 'as admin' do
        let(:valid_params) do
          { id: order1.id.to_s, data: { attributes: {
              payment_amout: 1000
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          patch 'update', valid_params
        end

        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end

      end
    end
  end
end