require 'rails_helper'

module Store
  describe Api::Store::ProductsController, type: :api do
    routes {Store::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:admin) { create(:administrator) }

    let!(:responsible) { create(:user) }
    let!(:user) { create(:user) }

    let!(:product1) { create(:product, user: admin, package: 'emails', price: 100) }
    let!(:product2) { create(:product, user: admin, package: 'files', price: 200) }
    let!(:product3) { create(:product, user: responsible, package: 'visio', price: 300) }
    let!(:product4) { create(:product, user: admin, package: 'whiteboard', price: 400) }

    let!(:customer1) { create(:customer) }
    let!(:customer2) { create(:customer) }

    let!(:order1) { create(:order, user: admin, writers: [responsible.id.to_s], product_ids: [product1.id, product2.id], customer: customer1) }

    describe 'GET /products' do
      context 'as admin' do
        before :each do
          authenticated_header(request, admin)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty

          expect(json['data'].length).to eq(4)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(4)
        end
      end
    end

    describe 'GET /products?ids' do
      before :each do
        authenticated_header(request, admin)
        get 'index', ids: [product1.id.to_s, product2.id.to_s]
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty

        expect(json['data'].length).to eq(2)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(2)
      end
    end

    describe 'GET /products?package' do
      before :each do
        authenticated_header(request, admin)
        get 'index', package: 'emails'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty

        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    describe 'GET /products/:id' do
      context 'as responsible' do
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: product3.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['id']).to eq(product3.id.to_s)
          expect(json['data']['attributes']['user_id']).to eq(responsible.id.to_s)
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['package']).to eq(product3.package)
        end
      end

      context 'as admin' do
        before :each do
          authenticated_header(request, admin)
          get 'show', id: product3.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['id']).to eq(product3.id.to_s)
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['package']).to eq(product3.package)
        end
      end

      context 'as user' do
        before :each do
          authenticated_header(request, user)
          get 'show', id: product3.id
        end

        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'POST /products without price' do
      context 'as admin' do
        let(:valid_params) do
          { data: { attributes: {
              title: 'cool',
              package: 'cool',
              body: 'brand new cool package'
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          post 'create', valid_params
        end

        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end
      end
    end

    describe 'POST /products with valid attrs' do
      context 'as admin' do
        let(:price) {153.2}
        let(:valid_params) do
          { data: { attributes: {
              title: 'cool',
              package: 'cool',
              body: 'brand new cool package',
              price: price
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          post 'create', valid_params
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(201)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['attributes']['read_level']).to eq('write')
          expect(json['data']['attributes']['user_id']).to eq(admin.id.to_s)
          expect(json['data']['attributes']['price']).to eq(price)
        end
      end
    end

    describe 'PATCH /products/:id' do
      context 'as admin' do
        let(:valid_params) do
          { id: product1.id.to_s, data: { attributes: {
              price: 1000,
              title: 'changed'
          } } }
        end

        before :each do
          authenticated_header(request, admin)
          patch 'update', valid_params
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns valid JSON' do
          subject
          expect(json['data']['attributes']['title']).to eq('changed')
          expect(json['data']['attributes']['price']).to eq(1000)
        end
      end
    end
  end
end