require 'rails_helper'

RSpec.describe(Store::Product, type: :model) do
  it 'has a valid factory' do
    expect(FactoryBot.create(:product, price: 0)).to be_valid
  end

  describe '#order basics' do
    let!(:user1) { create(:user) }
    let!(:product1) { create(:product, package: 'emails', price: 100) }
    let!(:customer) { create(:customer) }
    let!(:order) { create(:order, customer: customer, host: 'rec.plugandwork.fr') }

    it 'put products in order' do
      expect(order).to be_valid
      order.product_ids=[product1.id]
      order.save
      expect(order.reload.products.size).to eq 1

      order.payment_status='paid'
      order.save
      expect(order.reload.vouchers.size).to eq 1

      expect(order.vouchers.first.code).to_not be_blank
    end
  end

  describe '#product from app' do
    let!(:app) { create(:app, package: 'emails', type: 'view' ) }

    it 'has to be invalid without price' do
      product=app.to_product(nil)
      expect(product).to_not be_valid
    end

    it 'has to be valid with price' do
      product=app.to_product(1000)
      expect(product).to be_valid
      expect(product.price).to eq(1000)
      expect(product.title).to eq(app.name)
    end
  end

  describe '#order from app workflow for free products' do
    let!(:user1) { create(:user) }
    let!(:app) { create(:app, package: 'emails', type: 'view' ) }
    let!(:product1) { create(:product, title: '1 year emails', package: 'emails', price: 0) }
    let!(:product2) { create(:product, title: '1 year files', package: 'files', price: 0) }
    let!(:customer) { create(:customer) }
    let!(:order) { create(:order, customer: customer, host: 'rec.plugandwork.fr') }

    it 'put products in order and get vouchers' do
      expect(order).to be_valid
      order.product_ids=[product1.id, product2.id]
      order.save
      order.reload
      expect(order.products.size).to eq 2
      expect(order.total_price).to eq 0.0
      expect(order.payment_status).to eq 'paid'

      expect(order.vouchers.size).to eq 2

      expect(order.vouchers.first.code).to_not be_blank
    end
  end

  describe '#order from app workflow for products' do
    let!(:user1) { create(:user) }
    let!(:app) { create(:app, package: 'emails', type: 'view' ) }
    let!(:product1) { create(:product, title: '1 year emails', package: 'emails', price: 100) }
    let!(:product2) { create(:product, title: '1 year files', package: 'files', price: 200) }
    let!(:customer) { create(:customer) }
    let!(:order) { create(:order, customer: customer, host: 'rec.plugandwork.fr', product_ids: [product1.id, product2.id]) }

    it 'put products in order and get vouchers' do
      expect(order).to be_valid
      order.reload
      expect(order.products.size).to eq 2
      expect(order.payment_amount).to eq 0.0
      expect(order.total_price).to eq 300.0
      expect(order.payment_status).to eq 'not_processed'

      expect(order.vouchers.size).to eq 0

    end
  end
end
